# Generated by Django 2.2 on 2019-04-11 17:15

from django.db import migrations, models

from ..models import SearchQuery


class Migration(migrations.Migration):

    dependencies = [("elasticsearch_django", "0008_searchquery_search_terms")]

    operations = [
        migrations.AddField(
            model_name="searchquery",
            name="query_type",
            field=models.CharField(
                choices=SearchQuery.QueryType.choices,
                default="SEARCH",
                help_text="Does this query return results, or just the hit count?",
                max_length=10,
            ),
        )
    ]
