# Generated by Django 3.1 on 2020-08-15 11:52

import django.core.serializers.json
from django.db import migrations

from ..compat import JSONField


class Migration(migrations.Migration):

    dependencies = [
        ("elasticsearch_django", "0010_searchquery_total_hits_relation"),
    ]

    operations = [
        migrations.AddField(
            model_name="searchquery",
            name="aggregations",
            field=JSONField(
                default=dict,
                encoder=django.core.serializers.json.DjangoJSONEncoder,
                help_text="The raw aggregations returned from the query.",
            ),
        ),
    ]
